#include "hatpTaskList.hh"

using namespace std;

hatpTaskList::hatpTaskList(map<unsigned int, hatpNode*> n) {
	this->nodes = n;
	this->maxRank = -1;
	for(map<unsigned int, hatpNode*>::iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		this->computeNodeRank((*it).second);
	}
}

hatpTaskList::hatpTaskList(map<unsigned int, hatpNode*> n, vector<unsigned int> v) {
	for(unsigned int i = 0 ; i < v.size() ; i++) {
		this->nodes[v[i]] = n.find(v[i])->second;
	}
	this->maxRank = -1;
	for(map<unsigned int, hatpNode*>::iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		this->computeNodeRank((*it).second);
	}
}

hatpTaskList::~hatpTaskList() {}

size_t hatpTaskList::size() const{
	return this->nodes.size();
}

void hatpTaskList::computeNodeRank(hatpNode* cnode) {
	if(this->getRankOfNode(cnode->getID()) > -1) return;
	int cRank = -1;
	vector<unsigned int> pred = cnode->getTreePredecessors();
	for(unsigned int i = 0 ; i < pred.size() ; i++) {
		hatpNode* ctemp = this->nodes.find(pred[i])->second;
		this->computeNodeRank(ctemp);
		if(this->getRankOfNode(ctemp->getID()) > cRank) cRank = this->getRankOfNode(ctemp->getID());
	}
	this->ranks.push_back(make_pair(cRank + 1, cnode->getID()));
	if(this->maxRank < (cRank + 1)) this->maxRank = cRank + 1;
}

map<unsigned int, hatpNode*> hatpTaskList::getNodes() const {
	return this->nodes;
}

unsigned int hatpTaskList::getMaxRank() const {
	return this->maxRank;
}

int hatpTaskList::getRankOfNode(unsigned int n) const {
	for(unsigned int i = 0 ; i < this->ranks.size() ; i++) {
		if(this->ranks[i].second == n) return this->ranks[i].first;
	}
	return -1;
}

vector<hatpNode*> hatpTaskList::getNodesOfRank(unsigned int r) const {
	vector<hatpNode*> result;
	for(unsigned int i = 0 ; i < this->ranks.size() ; i++) {
		if(this->ranks[i].first == r) result.push_back(this->nodes.find(this->ranks[i].second)->second);
	}
	return result;	
}

bool hatpTaskList::isTaskListDecomposition(hatpTaskList* TL) const {
	bool result = true;
	map<unsigned int, hatpNode*> subTLnodes = TL->getNodes();
	for(map<unsigned int, hatpNode*>::const_iterator it = subTLnodes.begin() ; it != subTLnodes.end() ; it++) {
		if(this->nodes.find(it->second->getRootNode()) == this->nodes.end()) result = false;
	}
	return result;
}

hatpNode* hatpTaskList::getRootTaskListDecomposition(hatpTaskList* TL) const {
	map<unsigned int, hatpNode*> subTLnodes = TL->getNodes();
	for(map<unsigned int, hatpNode*>::const_iterator it = subTLnodes.begin() ; it != subTLnodes.end() ; it++) {
		if(this->nodes.find(it->second->getRootNode()) != subTLnodes.end()) return this->nodes.find(it->second->getRootNode())->second;
	}
	return NULL;
}

string hatpTaskList::toString() const {
	ostringstream result;
	result << "[";
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end(); it++) {
		result << (*it).first << " ";
	}
	result << "]";
	return result.str();
}
